package org.rcisoft.business.memLeaveMessage.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.rcisoft.business.memGoldCoinFlow.dao.MemGoldCoinFlowRepository;
import org.rcisoft.business.memGoldCoinFlow.entity.MemGoldCoinFlow;
import org.rcisoft.business.memInfo.dao.MemInfoRepository;
import org.rcisoft.business.memInfo.entity.MemInfo;
import org.rcisoft.business.memLeaveMessage.bean.MemLeaveMessageRedisBean;
import org.rcisoft.business.memLeaveMessage.dao.MemLeaveMessageRepository;
import org.rcisoft.business.memLeaveMessage.entity.*;
import org.rcisoft.business.memLeaveMessage.service.MemLeaveMessageService;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.model.CyPageInfo;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.service.impl.CyRedisServiceImpl;
import org.rcisoft.core.util.CyIdGenUtil;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.util.RedisCons;
import org.rcisoft.sys.dictionary.entity.DictData;
import org.rcisoft.sys.dictionary.service.impl.DictionaryServiceImpl;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import redis.clients.jedis.Jedis;
import redis.clients.jedis.params.ScanParams;
import redis.clients.jedis.resps.ScanResult;

import java.math.BigInteger;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * Created by cy on 2024年12月20日 下午3:35:34.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class MemLeaveMessageServiceImpl extends ServiceImpl<MemLeaveMessageRepository, MemLeaveMessage>  implements MemLeaveMessageService {



    @Autowired
    CyIdGenUtil cyIdGenUtil;

    @Autowired
    private MemLeaveMessageRepository memLeaveMessageRepository;

    @Autowired
    private DictionaryServiceImpl dictionaryService;

    @Autowired
    private StringRedisTemplate redisTemplate;

    @Autowired
    private CyRedisServiceImpl cyRedisServiceImpl;

    @Autowired
    private MemInfoRepository memInfoRepository;

    @Autowired
    private MemGoldCoinFlowRepository memGoldCoinFlowRepository;

    @Autowired
    private RedissonClient redissonClient;
    /**
     * 留言
     * @param memLeaveMessage
     * @return
     */
    @Override
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    public CyPersistModel leaveMessage(MemLeaveMessage memLeaveMessage) {
        //添加实名判断
        String userId = CyUserUtil.getAuthenBusinessId();
        MemInfo info = memInfoRepository.getInfoByUserId(userId);
        if (!"1".equals(info.getMemRealAuthen()))
            throw new CyServiceException("请先进行实名认证");
        //获取当前日期 例2025/2/20 获取02和20
        LocalDate today = LocalDate.now();
        DateTimeFormatter monthFormatter = DateTimeFormatter.ofPattern("MM");
        DateTimeFormatter dayFormatter = DateTimeFormatter.ofPattern("dd");
        // 格式化并获取月份和日期
        String month = today.format(monthFormatter);
        String day = today.format(dayFormatter);
        String date = month + day;
        //获取留言每天限制条数
        List<DictData> leaveMessageConfig = dictionaryService.selectByTypes("user_leave_message_config");
        DictData dictData1 = leaveMessageConfig.stream().filter(item -> item.getDictLabel().equals("leave_message_limit_count")).findFirst().orElse(null);
        DictData dictData2 = leaveMessageConfig.stream().filter(item -> item.getDictLabel().equals("pay_leave_message_limit_count")).findFirst().orElse(null);
        DictData dictData3 = leaveMessageConfig.stream().filter(item -> item.getDictLabel().equals("all_leave_message_limit_count")).findFirst().orElse(null);
        Integer leaveMessageLimitCount = Integer.valueOf(dictData1.getDictValue());//给每个人最多免费留言次数限制
        Integer payLeaveMessageLimitCount = Integer.valueOf(dictData2.getDictValue());//付费的留言次数限制
        Integer personalAllCount = leaveMessageLimitCount + payLeaveMessageLimitCount;//(个人)免费+付费 次数限制
        Integer allCount = Integer.valueOf(dictData3.getDictValue());//(所有人)总共留言次数限制
        //获取redis中已存储的限制次数
        Object redisLeaveMessageCount = cyRedisServiceImpl.hget(MemLeaveMessageRedisBean.USER_LEAVE_MESSAGE + date + ":" + userId, String.valueOf(memLeaveMessage.getTargetId()));
        Object redisLeaveMessageCountAll = cyRedisServiceImpl.get(MemLeaveMessageRedisBean.USER_LEAVE_MESSAGE_All + date + ":"  + userId);
        //1.进行总数对比
        if (redisLeaveMessageCountAll != null && (int)redisLeaveMessageCountAll <= allCount){
            //没超出总次数限制
            //2.进行判断对目标用户的留言总次数
            if (redisLeaveMessageCount != null && (int)redisLeaveMessageCount <= personalAllCount){
                //没超出对目标用户的留言总次数(免费+付费)
                //3.判断对目标用户的免费留言次数
                if (redisLeaveMessageCount != null && (int)redisLeaveMessageCount >= leaveMessageLimitCount) {
                    //超出 进行付费留言
                    //4.判断用户是否选择付费留言
                    if (memLeaveMessage.getIsPay().equals("1")){
                        //付费留言
                        //加redis锁
                        boolean isGetLock = false;
                        RLock lock = redissonClient.getLock(MemLeaveMessageRedisBean.USER_LEAVE_MESSAGE_LOCK + userId);
                        try {
                            isGetLock = lock.tryLock(Long.parseLong(MemLeaveMessageRedisBean.USER_LEAVE_MESSAGE_LOCK_WAIT_TIME),
                                    Long.parseLong(MemLeaveMessageRedisBean.USER_LEAVE_MESSAGE_LOCK_LEASE_TIME), TimeUnit.SECONDS);
                            if (isGetLock) {
                                //扣除该用户的金币数量
                                DictData dictData4 = leaveMessageConfig.stream().filter(item -> item.getDictLabel().equals("pay_count")).findFirst().orElse(null);
                                Integer payCount = Integer.valueOf(dictData4.getDictValue());
                                MemInfo memInfo = memInfoRepository.selectByOpenId(memLeaveMessage.getWxOpenid());
                                if (memInfo.getGoldCoinsCount() - payCount < 0){
                                    throw new CyServiceException(1003,"金币余额不足,请先充值");
                                }
                                memInfo.setGoldCoinsCount(memInfo.getGoldCoinsCount() - payCount);
                                memInfoRepository.updateById(memInfo);
                                //添加消费记录
                                MemGoldCoinFlow memGoldCoinFlow = new MemGoldCoinFlow();
                                memGoldCoinFlow.setType("2");
                                memGoldCoinFlow.setActionType("22");
                                memGoldCoinFlow.setCount(payCount);
                                memGoldCoinFlow.setEndCount(memInfo.getGoldCoinsCount());
                                memGoldCoinFlow.setTargetId(BigInteger.valueOf(memLeaveMessage.getTargetId()));
                                memGoldCoinFlowRepository.insert(memGoldCoinFlow);
                            }
                        } catch (InterruptedException e) {
                            Thread.currentThread().interrupt();
                            isGetLock = false;
                            throw new CyServiceException(1004,"服务器繁忙，请稍后再试");
                        } finally {
                            lock.unlock();
                            log.error("释放成功");
                        }
                    } else {
                        //不进行留言
                        throw new CyServiceException(1001,"今日对该用户留言次数超出限制");
                    }
                }
                //先插入到留言表
                memLeaveMessage.setCreateBy(Integer.valueOf(userId));
                memLeaveMessage.setUpdateBy(Integer.valueOf(userId));
                memLeaveMessage.setCreateDate(new Date());
                memLeaveMessage.setUpdateDate(new Date());
                memLeaveMessage.setReadStatus("0");
                int line = memLeaveMessageRepository.insert(memLeaveMessage);
                //修改对话表
                //查询是否已存在对话
                MemLeaveMessage memUserTalks = memLeaveMessageRepository.queryUserTalk(Integer.valueOf(userId),memLeaveMessage.getTargetId());
                MemLeaveMessage memUserTalk = new MemLeaveMessage();
                memUserTalk.setCreateBy(Integer.valueOf(userId));
                memUserTalk.setUpdateBy(Integer.valueOf(userId));
                memUserTalk.setTargetId(memLeaveMessage.getTargetId());
                memUserTalk.setContent(memLeaveMessage.getContent());
                memUserTalk.setCreateDate(new Date());
                memUserTalk.setUpdateDate(new Date());
                memUserTalk.setDelUser(0);
                int line1 = 0;
                if (memUserTalks != null){
                    //如果已经存在该对话，则只更新信息
                    memUserTalk.setBusinessId(memUserTalks.getBusinessId());
                    line1 = memLeaveMessageRepository.updateUserTalk(memUserTalk);
                } else {
                    //不存在该对话，则插入对话表
                    line1 = memLeaveMessageRepository.insertUserTalk(memUserTalk);
                }
                if (line1 > 0) {
                    //插入对话表成功，则更新redis
                    this.redisTemplate.opsForHash().increment(MemLeaveMessageRedisBean.USER_LEAVE_MESSAGE + date + ":"  + userId, String.valueOf(memLeaveMessage.getTargetId()), (double) 1L);
                    this.redisTemplate.opsForValue().increment(MemLeaveMessageRedisBean.USER_LEAVE_MESSAGE_All + date + ":"  + userId, (long) 1L);
                }
                return new CyPersistModel(line);
            } else if (redisLeaveMessageCount == null){
                //今日第一次对该用户进行留言
                //先插入到留言表
                memLeaveMessage.setCreateBy(Integer.valueOf(userId));
                memLeaveMessage.setUpdateBy(Integer.valueOf(userId));
                memLeaveMessage.setCreateDate(new Date());
                memLeaveMessage.setUpdateDate(new Date());
                memLeaveMessage.setReadStatus("0");
                int line = memLeaveMessageRepository.insert(memLeaveMessage);
                //查询是否已存在对话
                MemLeaveMessage memUserTalks = memLeaveMessageRepository.queryUserTalk(Integer.valueOf(userId),memLeaveMessage.getTargetId());
                //如果已经存在该对话，则只更新信息
                MemLeaveMessage memUserTalk = new MemLeaveMessage();
                memUserTalk.setCreateBy(Integer.valueOf(userId));
                memUserTalk.setUpdateBy(Integer.valueOf(userId));
                memUserTalk.setTargetId(memLeaveMessage.getTargetId());
                memUserTalk.setContent(memLeaveMessage.getContent());
                memUserTalk.setCreateDate(new Date());
                memUserTalk.setUpdateDate(new Date());
                //留言未删除默认值为0
                memUserTalk.setDelUser(0);
                if (memUserTalks != null){
                    //已经给该用户留过言
                    memUserTalk.setBusinessId(memUserTalks.getBusinessId());
                    int line1 = memLeaveMessageRepository.updateUserTalk(memUserTalk);
                    if (line1 > 0) {
                        //插入对话表成功，则更新redis
                        cyRedisServiceImpl.hset(MemLeaveMessageRedisBean.USER_LEAVE_MESSAGE + date + ":"  + userId, String.valueOf(memLeaveMessage.getTargetId()), 1L);
                        this.redisTemplate.opsForValue().increment(MemLeaveMessageRedisBean.USER_LEAVE_MESSAGE_All + date + ":"  + userId, (long) 1L);
                    }
                    return new CyPersistModel(line1);
                } else {
                    //第一次给该用户进行留言
                    int line1 = memLeaveMessageRepository.insertUserTalk(memUserTalk);
                    if (line1 > 0) {
                        //插入对话表成功，则更新redis
                        cyRedisServiceImpl.hset(MemLeaveMessageRedisBean.USER_LEAVE_MESSAGE + date + ":"  + userId, String.valueOf(memLeaveMessage.getTargetId()), 1L);
                        this.redisTemplate.opsForValue().increment(MemLeaveMessageRedisBean.USER_LEAVE_MESSAGE_All + date + ":"  + userId, (long) 1L);
                    }
                }
                return new CyPersistModel(line);
            } else {
                throw new CyServiceException(1001,"今日对该用户留言次数超出限制");
            }
        } else if (redisLeaveMessageCountAll == null){
            //当天第一次进行留言
            //先插入到留言表
            memLeaveMessage.setCreateBy(Integer.valueOf(userId));
            memLeaveMessage.setUpdateBy(Integer.valueOf(userId));
            memLeaveMessage.setCreateDate(new Date());
            memLeaveMessage.setUpdateDate(new Date());
            memLeaveMessage.setReadStatus("0");
            memLeaveMessageRepository.insert(memLeaveMessage);
            //修改对话表
            //查询是否已存在对话
            MemLeaveMessage memUserTalks = memLeaveMessageRepository.queryUserTalk(Integer.valueOf(userId),memLeaveMessage.getTargetId());
            MemLeaveMessage memUserTalk = new MemLeaveMessage();
            memUserTalk.setCreateBy(Integer.valueOf(userId));
            memUserTalk.setUpdateBy(Integer.valueOf(userId));
            memUserTalk.setCreateDate(new Date());
            memUserTalk.setUpdateDate(new Date());
            memUserTalk.setTargetId(memLeaveMessage.getTargetId());
            memUserTalk.setContent(memLeaveMessage.getContent());
            //留言未删除默认值为0
            memUserTalk.setDelUser(0);
            int line1 = 0;
            if (memUserTalks != null){
                //如果已经存在该对话，则只更新信息
                memUserTalk.setBusinessId(memUserTalks.getBusinessId());
                line1 = memLeaveMessageRepository.updateUserTalk(memUserTalk);
            } else {
                line1 = memLeaveMessageRepository.insertUserTalk(memUserTalk);
            }
            //插入对话表成功，则更新redis
            if ( line1 > 0 ){
                cyRedisServiceImpl.hset(MemLeaveMessageRedisBean.USER_LEAVE_MESSAGE + date + ":"  + userId, String.valueOf(memLeaveMessage.getTargetId()), 1L);
                cyRedisServiceImpl.set(MemLeaveMessageRedisBean.USER_LEAVE_MESSAGE_All + date + ":"  + userId,1L, Long.valueOf(48 * 60 * 60));
                cyRedisServiceImpl.expire(MemLeaveMessageRedisBean.USER_LEAVE_MESSAGE + date + ":"  + userId, 48 * 60 * 60); // 24小时
            }
            return new CyPersistModel(line1);
        } else {
            //超出总次数限制
            throw new CyServiceException(1002,"今日留言次数超出限制");
        }
    }

    /**
     * 分页查询对话列表
     *
     * @param paginationUtility
     * @param memLeaveMessage
     */
    @Override
    public IPage<MemLeaveMessage> queryTalkByPagination(CyPageInfo<MemLeaveMessage> paginationUtility, MemLeaveMessage memLeaveMessage) {
        memLeaveMessage.setCreateBy(Integer.valueOf(CyUserUtil.getAuthenBusinessId()));
        IPage<MemLeaveMessage> userLeaveMessageIPage = memLeaveMessageRepository.queryTalkByPagination(paginationUtility, memLeaveMessage);
        List<MemLeaveMessage> talks = userLeaveMessageIPage.getRecords();
        talks.forEach(item->{
            //查询每一个对话的我的未读留言数量
            //1.查询该对话的所有留言 并且计算出当前用户对该留言的未读信息数量 对话的createBy和targetId会随时变 所以当前人id要手动获取
            int count1 = memLeaveMessageRepository.queryTalkNoReadCount(item.getCreateBy(),item.getTargetId(),Integer.valueOf(CyUserUtil.getAuthenBusinessId()));
            //查询互换微信未回应数量
            int count2 = memInfoRepository.queryTalkWeChatTaskNoReplyCount(item.getCreateBy(),item.getTargetId(),Integer.valueOf(CyUserUtil.getAuthenBusinessId()));
            item.setNoReadCount(count1 + count2);
        });
        return userLeaveMessageIPage;
    }

    /**
     * 查询留言列表详情 1对1的
     * @param paginationUtility
     * @param memLeaveMessage
     */
    @Override
    public IPage<MemLeaveMessage> leaveMessageDetailByPagination(CyPageInfo<MemLeaveMessage> paginationUtility, MemLeaveMessage memLeaveMessage) {
        memLeaveMessage.setCreateBy(Integer.valueOf(CyUserUtil.getAuthenBusinessId()));
        //先按倒序取最新的十条
        IPage<MemLeaveMessage> userLeaveMessageIPage = memLeaveMessageRepository.leaveMessageDetailList(paginationUtility, memLeaveMessage);
        //修改留言未读状态
        memLeaveMessageRepository.updateMessageReadStatus(Integer.valueOf(CyUserUtil.getAuthenBusinessId()),memLeaveMessage.getTargetId(),"1");
        return userLeaveMessageIPage;
    }

    /**
     * 分页查询留言列表
     *
     * @param paginationUtility
     * @param memLeaveMessage
     */
    @Override
    public IPage<MemLeaveMessage> queryMessageByPagination(CyPageInfo<MemLeaveMessage> paginationUtility, MemLeaveMessage memLeaveMessage) {
        IPage<MemLeaveMessage> userLeaveMessageIPage = memLeaveMessageRepository.queryMessageByPagination(paginationUtility, memLeaveMessage);
        return userLeaveMessageIPage;
    }


    @Override
    public CyPersistModel deleteMessage(MemLeaveMessage memLeaveMessage) {
        if (memLeaveMessageRepository.selectDelUserById(memLeaveMessage) == 0){
            memLeaveMessage.setDelUser(Integer.valueOf(CyUserUtil.getAuthenBusinessId()));
            int line = memLeaveMessageRepository.updateTalkDel(memLeaveMessage);
            return new CyPersistModel(line);
        } else {
            int line = memLeaveMessageRepository.deleteTalkById(memLeaveMessage);
            return new CyPersistModel(line);
        }
    }

    @Override
    public int noReadMessageAllCount() {
        //获取当前登录人
        String userId = CyUserUtil.getAuthenBusinessId();
        //查询未读的留言数
        int count1 = memLeaveMessageRepository.leaveMessageNoReadAllCount(Integer.valueOf(userId));
        //查询互换微信未回应数量
        int count2 = memInfoRepository.queryWeChatTaskNoReplyCount(userId);
        //查询未读的要微信请求数
        return count1 + count2;
    }
}
